"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.SharedDevice = void 0;
// @ts-ignore
const miio_1 = __importDefault(require("miio"));
const lodash_debounce_1 = __importDefault(require("lodash.debounce"));
const queue_1 = require("./queue");
const foundation_1 = require("./foundation");
const MIoTDevice_utils_1 = require("../MIoTDevice.utils");
class Device {
    constructor() {
        this.device = {};
        this.pool = {};
        this.isConnected = (identify) => {
            foundation_1.SharedFoundation.log.debug(`SHARED DEVICE CHECKING CONNECTING ${identify.name} ${identify.address}`, Date.now());
            const deviceInstance = this.device[identify.address];
            if (!deviceInstance)
                return false;
            const now = Date.now();
            const diff = now - deviceInstance.timeout;
            deviceInstance.timeout = now;
            return diff < foundation_1.SharedFoundation.config.reConnectThreshold;
        };
        this.requestConnect = async (identify) => {
            // Do connect
            try {
                // Log
                foundation_1.SharedFoundation.log.info(`${identify.name} ${identify.address} start ${!!this.device[identify.address] ? 're-' : ''}connecting.`);
                // Create device instance
                const device = await miio_1.default.device({
                    address: identify.address,
                    token: identify.token
                });
                device.did = MIoTDevice_utils_1.getDeviceId(device.id);
                device.timeout = Date.now();
                // Update
                this.device[identify.address] = device;
                // Resolve queues
                queue_1.SharedQueue.resolve(identify, device);
                // Log
                foundation_1.SharedFoundation.log.info(`${identify.name} ${identify.address} connected.`);
            }
            catch (e) {
                // Retry if failure
                if (this.isConnected(identify))
                    return;
                foundation_1.SharedFoundation.log.info(`${identify.name} ${identify.address} ${identify.token} connect failure, reconnecting ...`);
                foundation_1.SharedFoundation.log.error(e);
                await MIoTDevice_utils_1.sleep(foundation_1.SharedFoundation.config.reConnectFailureThreshold);
                await this.requestConnect(identify);
            }
        };
    }
    debounceRequestConnect(identify) {
        if (typeof this.pool[identify.address] !== 'function') {
            this.pool[identify.address] = lodash_debounce_1.default(() => this.requestConnect(identify), foundation_1.SharedFoundation.config.requestConnectDebounceThreshold);
        }
        this.pool[identify.address]();
    }
    getInstance(identify) {
        foundation_1.SharedFoundation.log.debug(`SHARED DEVICE GETTING INSTANCE ${identify.name} ${identify.address}`, Date.now());
        // return this.device[identify]
        return new Promise(resolve => {
            // Queue update
            queue_1.SharedQueue.append(identify, resolve);
            // Trigger getter
            this.debounceRequestConnect(identify);
        });
    }
}
exports.SharedDevice = new Device();
//# sourceMappingURL=device.js.map